// Import any other script files here, e.g.:
// import * as myModule from "./mymodule.js";
import AudioManager from "./audioManager.js";
import EnemyManager from "./enemyManager.js"

//Shooting
let shootingDelay = 0;
let reloadBegin = 0;
let reloadRotated = 0;

//Animations
let previousYVelocity = 0;
let yVelocity = 0;

//Sounds
let audioManager = null;
let shoot = null;
let jump = null;
let walk = null;
let hurt1 = null;
let hurt2 = null;

//Enemies
let enemyManager = null;

//Player
let healthBar = null;
let player = null;
let shotgun = null;
let physics = null;
let groundDetection = null;

runOnStartup(async runtime =>
{
	// Code to run on the loading screen.
	// Note layouts, objects etc. are not yet available.
	runtime.addEventListener("beforeprojectstart", () => OnBeforeProjectStart(runtime));
	
	// Initialise the audio manager. See AudioManager.js for details.
	audioManager = new AudioManager(runtime);
	
	//Init Enemy Manager
	enemyManager = new EnemyManager(runtime);
	
	//Assign audio files
	[shoot, jump, walk, hurt1, hurt2] = await Promise.all([
		audioManager.loadSound("shoot.webm"),
		audioManager.loadSound("jump.webm"),
		audioManager.loadSound("walk.webm"),
		audioManager.loadSound("hurt.webm"),
		audioManager.loadSound("hurt2.webm"),
	]);
});

async function OnBeforeProjectStart(runtime)
{
	// Code to run just before 'On start of layout' on
	// the first layout. Loading has finished and initial
	// instances are created and available to use here.
	
	runtime.addEventListener("tick", () => Tick(runtime));
	
	//Sets the player spawn for specific layouts
	runtime.getLayout("Hub").addEventListener("beforelayoutstart", () => spawnPlayer(runtime, 100, 100));
	//Gives the Hub level the Initialization function
	runtime.getLayout("Hub").addEventListener("beforelayoutstart", () => Initialization(runtime));
	
	//Gives the Death and Win screens the reset function
	runtime.getLayout("Death").addEventListener("beforelayoutstart", () => Reset());
	runtime.getLayout("Win").addEventListener("beforelayoutstart", () => Reset());
}

function Reset()
{
	healthBar = null;
	player = null;
	groundDetection = null;
	shotgun = null;
}

function Initialization(runtime)
{
	healthBar = runtime.objects.HealthBar.getFirstInstance();
	player = runtime.objects.Player.getFirstInstance();
	groundDetection = runtime.objects.GroundDetection.getFirstInstance();
	shotgun = runtime.objects.Shotgun.getFirstInstance();
	
	player.addEventListener("framechange", () => playerWalk(runtime))
	physics = player.behaviors.Physics;
	physics.isPreventRotation = true;
	physics.linearDamping = 3;
	physics.behavior.worldGravity = 50;
}

function Tick(runtime)
{
	//Ensures we don't run in to any errors
	if (!player) return;
	
	//Enemy movement
	enemyManager.move(player);

	previousYVelocity = yVelocity;
	yVelocity = physics.getVelocityY();

	//Reset jump variable if the player begins moving up
	if (previousYVelocity > 0 && yVelocity < 0) player.instVars["jump"] = 0;
	
	//Reset fall variable if the player begins moving down
	if (previousYVelocity < 0 && yVelocity > 0) player.instVars["fall"] = 0;
	
	//Disables jumping variable when the player is falling
	if (yVelocity > 0) player.instVars["jumping"] = false;
	
	//Shooting Delay
	if (shootingDelay > 0) shootingDelay -= runtime.dt;
	
	//Health bar
	let barValue = player.instVars["health"];
	if (barValue < 0) barValue = 0;
	healthBar.setAnimation(barValue.toString());
	
	//Player Death
	if (player.instVars["health"] <= 0 && !player.instVars["died"])
	{
		console.log("Player is dead.");
		player.instVars["health"] = 0;
		player.instVars["died"] = true;
		enemyManager.killAll();
		shotgun.destroy();
	}
	
	if (player.instVars["health"] == 0 && player.instVars["hurtTime"] <= 0 && player.animationName != "Death") player.setAnimation("Death");
	
	if (player.instVars["died"])
	{
		player.instVars["hurtTime"] -= runtime.dt;
		return;
	}
	
	//Player movement
	if (player.instVars["hurtTime"] <= 0)
	{
		if (runtime.keyboard.isKeyDown("KeyW") && player.instVars["grounded"] && !player.instVars["jumping"])
		{
			physics.applyImpulse(0,-15);
			player.instVars["jumping"] = true;
			audioManager.playSound(jump);
			
			//Dust
			const dust = runtime.objects.JumpDust.createInstance(0, player.x, player.y, "");
		}

		if (runtime.keyboard.isKeyDown("KeyA")) physics.applyForce(-50,0);

		if (runtime.keyboard.isKeyDown("KeyD")) physics.applyForce(50,0);
	
	}
	else player.instVars["hurtTime"] -= runtime.dt;
	
	//Ground Detection
	if (player) groundDetection.setPosition(player.x, player.y + player.height / 2);
	
	//Player shoot
	if (runtime.mouse.isMouseButtonDown(0) && player.instVars["ammo"] > 0 && shootingDelay <= 0)
	{
		//Physics
		physics.applyImpulseTowardPosition(-20, runtime.mouse.getMouseX(), runtime.mouse.getMouseY());
		player.instVars["ammo"] -= 1;
		shootingDelay = 0.5;
		
		//Muzzle Flash
		const flash = runtime.objects.MuzzleFlash.createInstance(0, shotgun.x, shotgun.y, "");
		flash.angle = shotgun.angle;
		flash.setPosition(flash.x + Math.cos(flash.angle) * 58, flash.y + Math.sin(flash.angle) * 58);
		flash.setSize(64, 64);
		
		//Sound
		audioManager.playSound(shoot);
		
		//Bullet Spray
		for(let i = 0; i < 8; i++)
		{
			const bullet = runtime.objects.Bullet.createInstance(0, shotgun.x, shotgun.y, "");
			bullet.angle = shotgun.angle + getRandom(-0.15, 0.15);
			bullet.behaviors.Bullet.speed = 1300;
		
			bullet.isVisible = true;
			bullet.width = 80;
			bullet.height = 25;
		}
	}
	
	//Shotgun Flip
	if (shotgun.angleDegrees <= 90 || shotgun.angleDegrees >= 270) shotgun.height = 64;
	else shotgun.height = -64;
	
	//Player Reload
	if (player.instVars["ammo"] == 0 && player.instVars["reloading"] == 0 && reloadRotated != 360) player.instVars["reloading"] = 1;
	else
	{
		shotgun.angle
	}
	
	if (player.instVars["reloading"] == 1)
	{
		shotgun.angle += 0.125;
		reloadRotated += 8;
	}
	
	if (player.instVars["ammo"] == 0 && player.instVars["reloading"] == 1 && reloadRotated == 360)
	{
		 player.instVars["reloading"] = 0
		 player.instVars["ammo"] = 2;
		 reloadRotated = 0;
	}
}

function getRandom(min, max)
{
  return Math.random() * (max - min) + min;
}

function spawnPlayer(runtime, x, y)
{
	const p = runtime.objects.Player.createInstance(0, x, y, "")
	p.setSize(64, 64);
	p.instVars["ammo"] = 2;
	p.instVars["health"] = 7;
	
	const s = runtime.objects.Shotgun.createInstance(0, x, y, "");
	s.setSize(64, 64);
	
	const g = runtime.objects.GroundDetection.createInstance(0, x, y, "");
	g.setSize(8, 10);
}

function playerWalk(runtime)
{
	if (player.animationName == "Run" && (player.animationFrame == 6 || player.animationFrame == 14))
	{
		const dust = runtime.objects.RunDust.createInstance(0, player.x, player.y, "");
		if (player.instVars["mirrored"]) dust.width = -dust.width;
		audioManager.playSound(walk);
	}
}

export function playerHurt()
{
	audioManager.playSound(hurt2);
}

export function playerCritical()
{
	audioManager.playSound(hurt1);
}